{% from "_forms.html" import quickform %}
{% extends "layout.html" %}
{% block content %}

{{ quickform(unlockForm) }}

<div>The local wallet is encrypted with your password, and contains any private keys belonging to your accounts.</div>
{% endblock %}