{% from "_macros.html" import render_int_string %}
<div class="item">
	<i class="asterisk icon"></i>
	<div class="content">
		<div class="header">Betting market group update {{ operationId }}</div>
		<div class="description">
		<a href="{{ url_for('overview', typeName='bettingmarketgroup', identifier=data['betting_market_group_id']) }}">
			id={{ data['betting_market_group_id'] }}
		</a> <br />
		{% if data['new_status'] %}Status: {{ data['new_status'] }}{% endif %}
		{% if data['status'] %}Status: {{ data['status'] }}{% endif %}
		{% if data['new_event_id'] %}Event: {{ data['new_event_id'] }}{% endif %} 
		{% if data['new_rules_id'] %}Rule: {{ data['new_rules_id'] }}{% endif %}  	
		{% if data['new_asset_id'] %}Asset: {{ data['new_asset_id'] }}{% endif %}<br />
		{% if data['new_description'] %}Description: {{ render_int_string(data['new_description']) }}{% endif %}<br />
		</div>
	</div>
</div>