{% from "_forms.html" import quickform, parsefield %}
{% extends "layout.html" %}

{% block content %}
<table style="width: 100%"><tbody><tr><td>
<h2>{{ formTitle }}{% if form and form.title %}{{ form.title }}{% endif %}</h2>
</td></tr></tbody></table><br />

{{ data }}

<pre>
{{ preformatted_string }}
</pre>

{% if form %}
	{{ quickform(form) }}
{% endif %}

{{ formMessage }}

{% if formMessages %}
{% for message in formMessages %}
	{{ message }}<br />
{% endfor %}
{% endif %}

{% endblock %}