{% from "_macros.html" import render_int_string %}
<div class="item">
	<i class="asterisk icon"></i>
	<div class="content">
		<div class="header">Sport update {{ operationId }}</div>
		<div class="description">
		<a href="{{ url_for('overview', typeName='sport', identifier=data['sport_id']) }}">
			id={{ data['sport_id'] }}
		</a> <br />
		{% if data['new_name'] %}Name: {{ render_int_string(data['new_name']) }}{% endif %}
		</div>
	</div>
</div>