{% from "_macros.html" import render_int_string %}
<div class="item">
	<i class="asterisk icon"></i>
	<div class="content">
		<div class="header">Event group creation {{ operationId }}</div>
		<div class="description">
		Sport: {{ data['sport_id'] }}  <br/>
		Name: {{ render_int_string(data['name']) }}
		</div>
	</div>
</div>