{% from "_macros.html" import render_int_string %}
<div class="item">
	<i class="asterisk icon"></i>
	<div class="content">
		<div class="header">Betting market group creation {{ operationId }}</div>
		<div class="description">
		{% if data['status'] %}Status: {{ data['status'] }}{% endif %}
		{% if data['event_id'] %}Event: {{ data['event_id'] }}{% endif %} 
		{% if data['rules_id'] %}Rule: {{ data['rules_id'] }}{% endif %}  	
		{% if data['asset_id'] %}Asset: {{ data['asset_id'] }}{% endif %} 	</br>
		{% if data['description'] %}Description: {{ render_int_string(data['description']) }}{% endif %}<br/>
		</div>
	</div>
</div>