{% from "_macros.html" import render_int_string %}
<div class="item">
	<i class="asterisk icon"></i>
	<div class="content">
		<div class="header">Event creation {{ operationId }}</div>
		<div class="description">
		{% if data['status'] %}Status: {{ data['status'] }}{% endif %}		
		{% if data['event_group_id'] %}Event group: {{ data['event_group_id'] }}{% endif %} <br/>	
		{% if data['name'] %}Name: {{ render_int_string(data['name']) }}{% endif %} <br/>	
		{% if data['season'] %}Season: {{ render_int_string(data['season']) }}{% endif %} <br/>	
		{% if data['start_time'] %}Start time: {{ data['start_time'] }}Z{% endif %}</div>
	</div>
</div>