body {
 background-color: #FFFFFF;
}
.ui.menu img.logo {
 height: 64px;
}
.main.container {
 margin-top: 3em;
}
.ui.footer.segment {
 margin: 5em 0em 0em;
 padding: 5em 0em;
}
.header.branding img {
 height: 50px;
}
.ui.vertical.footer.segment {
 border: 0px;
}
